@@ -10,14 +10,20 @@ for signing and checking OAuth 1.0 RFC 5849 requests.
 """
 
 import logging
+import sys
 import time
 try:
     import urlparse
 except ImportError:
     import urllib.parse as urlparse
 
+if sys.version_info[0] == 3:
+    bytes_type = bytes
+else:
+    bytes_type = str
+
 from oauthlib.common import Request, urlencode, generate_nonce
-from oauthlib.common import generate_timestamp
+from oauthlib.common import generate_timestamp, to_unicode
 from . import parameters, signature, utils
 
 SIGNATURE_HMAC = "HMAC-SHA1"
@@ -41,17 +47,48 @@ class Client(object):
             callback_uri=None,
             signature_method=SIGNATURE_HMAC,
             signature_type=SIGNATURE_TYPE_AUTH_HEADER,
-            rsa_key=None, verifier=None, realm=None):
-        self.client_key = client_key
-        self.client_secret = client_secret
-        self.resource_owner_key = resource_owner_key
-        self.resource_owner_secret = resource_owner_secret
-        self.signature_method = signature_method
-        self.signature_type = signature_type
-        self.callback_uri = callback_uri
-        self.rsa_key = rsa_key
-        self.verifier = verifier
-        self.realm = realm
+            rsa_key=None, verifier=None, realm=None,
+            encoding='utf-8', decoding=None,
+            nonce=None, timestamp=None):
+        """Create an OAuth 1 client.
+
+        :param client_key: Client key (consumer key), mandatory.
+        :param resource_owner_key: Resource owner key (oauth token).
+        :param resource_owner_secret: Resource owner secret (oauth token secret).
+        :param callback_uri: Callback used when obtaining request token.
+        :param signature_method: SIGNATURE_HMAC, SIGNATURE_RSA or SIGNATURE_PLAINTEXT.
+        :param signature_type: SIGNATURE_TYPE_AUTH_HEADER (default),
+                               SIGNATURE_TYPE_QUERY or SIGNATURE_TYPE_BODY
+                               depending on where you want to embed the oauth
+                               credentials.
+        :param rsa_key: RSA key used with SIGNATURE_RSA.
+        :param verifier: Verifier used when obtaining an access token.
+        :param realm: Realm (scope) to which access is being requested.
+        :param encoding: If you provide non-unicode input you may use this
+                         to have oauthlib automatically convert.
+        :param decoding: If you wish that the returned uri, headers and body
+                         from sign be encoded back from unicode, then set
+                         decoding to your preferred encoding, i.e. utf-8.
+        :param nonce: Use this nonce instead of generating one. (Mainly for testing)
+        :param timestamp: Use this timestamp instead of using current. (Mainly for testing)
+        """
+        # Convert to unicode using encoding if given, else assume unicode
+        encode = lambda x: to_unicode(x, encoding) if encoding else x
+
+        self.client_key = encode(client_key)
+        self.client_secret = encode(client_secret)
+        self.resource_owner_key = encode(resource_owner_key)
+        self.resource_owner_secret = encode(resource_owner_secret)
+        self.signature_method = encode(signature_method)
+        self.signature_type = encode(signature_type)
+        self.callback_uri = encode(callback_uri)
+        self.rsa_key = encode(rsa_key)
+        self.verifier = encode(verifier)
+        self.realm = encode(realm)
+        self.encoding = encode(encoding)
+        self.decoding = encode(decoding)
+        self.nonce = encode(nonce)
+        self.timestamp = encode(timestamp)
 
         if self.signature_method == SIGNATURE_RSA and self.rsa_key is None:
             raise ValueError('rsa_key is required when using RSA signature method.')
@@ -97,9 +134,13 @@ class Client(object):
     def get_oauth_params(self):
         """Get the basic OAuth parameters to be used in generating a signature.
         """
+        nonce = (generate_nonce()
+                 if self.nonce is None else self.nonce)
+        timestamp = (generate_timestamp()
+                     if self.timestamp is None else self.timestamp)
         params = [
-            ('oauth_nonce', generate_nonce()),
-            ('oauth_timestamp', generate_timestamp()),
+            ('oauth_nonce', nonce),
+            ('oauth_timestamp', timestamp),
             ('oauth_version', '1.0'),
             ('oauth_signature_method', self.signature_method),
             ('oauth_consumer_key', self.client_key),
@@ -154,7 +195,9 @@ class Client(object):
 
         Returns a 3-tuple of the signed request's URI, headers, and body.
         Note that http_method is not returned as it is unaffected by the OAuth
-        signing process.
+        signing process. Also worth noting is that duplicate parameters
+        will be included in the signature, regardless of where they are
+        specified (query, body).
 
         The body argument may be a dict, a list of 2-tuples, or a formencoded
         string. The Content-Type header must be 'application/x-www-form-urlencoded'
@@ -168,11 +211,16 @@ class Client(object):
         If the body does contain parameters, it will be returned as a properly-
         formatted formencoded string.
 
-        All string data MUST be unicode. This includes strings inside body
-        dicts, for example.
+        Body may not be included if the http_method is either GET or HEAD as
+        this changes the semantic meaning of the request.
+
+        All string data MUST be unicode or be encoded with the same encoding
+        scheme supplied to the Client constructor, default utf-8. This includes
+        strings inside body dicts, for example.
         """
         # normalize request data
-        request = Request(uri, http_method, body, headers)
+        request = Request(uri, http_method, body, headers,
+                          encoding=self.encoding)
 
         # sanity check
         content_type = request.headers.get('Content-Type', None)
@@ -208,6 +256,15 @@ class Client(object):
                 should_have_params and has_params and not multipart):
             raise ValueError('Body signatures may only be used with form-urlencoded content')
 
+        # We amend http://tools.ietf.org/html/rfc5849#section-3.4.1.3.1
+        # with the clause that parameters from body should only be included
+        # in non GET or HEAD requests. Extracting the request body parameters
+        # and including them in the signature base string would give semantic
+        # meaning to the body, which it should not have according to the
+        # HTTP 1.1 spec.
+        elif http_method.upper() in ('GET', 'HEAD') and has_params:
+            raise ValueError('GET/HEAD requests should not include body.')
+
         # generate the basic OAuth parameters
         request.oauth_params = self.get_oauth_params()
 
@@ -215,8 +272,18 @@ class Client(object):
         request.oauth_params.append(('oauth_signature', self.get_oauth_signature(request)))
 
         # render the signed request and return it
-        return self._render(request, formencode=True,
-                            realm=(realm or self.realm))
+        uri, headers, body = self._render(request, formencode=True,
+                realm=(realm or self.realm))
+
+        if self.decoding:
+            logging.debug('Encoding URI, headers and body to %s.', self.decoding)
+            uri = uri.encode(self.decoding)
+            body = body.encode(self.decoding) if body else body
+            new_headers = {}
+            for k, v in headers.items():
+                new_headers[k.encode(self.decoding)] = v.encode(self.decoding)
+            headers = new_headers
+        return uri, headers, body
 
 
 class Server(object):
@@ -257,10 +324,10 @@ class Server(object):
     each methods documentation for detailed usage.
     The following methods must be implemented:
 
-    - validate_client
+    - validate_client_key
     - validate_request_token
     - validate_access_token
-    - validate_nonce_and_timestamp
+    - validate_timestamp_and_nonce
     - validate_redirect_uri
     - validate_requested_realm
     - validate_realm
@@ -578,7 +645,14 @@ class Server(object):
         straightforward for the provider to verify whether the supplied
         redirect_uri is valid or not.
 
+        Alternatively per `Section 2.1`_ of the spec:
+
+        "If the client is unable to receive callbacks or a callback URI has
+        been established via other means, the parameter value MUST be set to
+        "oob" (case sensitive), to indicate an out-of-band configuration."
+
         .. _`CWE-601`: http://cwe.mitre.org/top25/index.html#CWE-601
+        .. _`Section 2.1`: https://tools.ietf.org/html/rfc5849#section-2.1
         """
         raise NotImplementedError("Subclasses must implement this function.")
 
@@ -615,9 +689,30 @@ class Server(object):
         """
         raise NotImplementedError("Subclasses must implement this function.")
 
+    def verify_request_token_request(self, uri, http_method='GET', body=None,
+            headers=None):
+        """Verify the initial request in the OAuth workflow.
+
+        During this step the client obtains a request token for use during
+        resource owner authorization (which is outside the scope of oauthlib).
+        """
+        return self.verify_request(uri, http_method=http_method, body=body,
+                headers=headers, require_resource_owner=False,
+                require_realm=True, require_callback=True)
+
+    def verify_access_token_request(self, uri, http_method='GET', body=None,
+            headers=None):
+        """Verify the second request in the OAuth workflow.
+
+        During this step the client obtains the access token for use when
+        accessing protected resources.
+        """
+        return self.verify_request(uri, http_method=http_method, body=body,
+                headers=headers, require_verifier=True)
+
     def verify_request(self, uri, http_method='GET', body=None,
             headers=None, require_resource_owner=True, require_verifier=False,
-            require_realm=False, required_realm=None):
+            require_realm=False, required_realm=None, require_callback=False):
         """Verifies a request ensuring that the following is true:
 
         Per `section 3.2`_ of the spec.
@@ -747,6 +842,9 @@ class Server(object):
         if require_verifier and not self.check_verifier(request.verifier):
             raise ValueError("Invalid verifier.")
 
+        if require_callback and not request.callback_uri:
+            raise ValueError("Missing callback URI.")
+
         # Servers receiving an authenticated request MUST validate it by:
         #   If using the "HMAC-SHA1" or "RSA-SHA1" signature methods, ensuring
         #   that the combination of nonce/timestamp/token (if present)
@@ -774,11 +872,16 @@ class Server(object):
         # Note that early exit would enable client enumeration
         valid_client = self.validate_client_key(request.client_key)
         if not valid_client:
-            client_key = self.dummy_client
+            request.client_key = self.dummy_client
 
-        # Ensure a valid redirection uri is used
-        valid_redirect = self.validate_redirect_uri(request.client_key,
-                request.callback_uri)
+        # Callback is normally never required, except for requests for
+        # a Temporary Credential as described in `Section 2.1`_
+        # .._`Section 2.1`: http://tools.ietf.org/html/rfc5849#section-2.1
+        if require_callback:
+            valid_redirect = self.validate_redirect_uri(request.client_key,
+                    request.callback_uri)
+        else:
+            valid_redirect = True
 
         # The server SHOULD return a 401 (Unauthorized) status code when
         # receiving a request with invalid or expired token.
@@ -791,11 +894,13 @@ class Server(object):
             if require_verifier:
                 valid_resource_owner = self.validate_request_token(
                     request.client_key, request.resource_owner_key)
+                if not valid_resource_owner:
+                    request.resource_owner_key = self.dummy_request_token
             else:
                 valid_resource_owner = self.validate_access_token(
                     request.client_key, request.resource_owner_key)
-            if not valid_resource_owner:
-                resource_owner_key = self.dummy_resource_owner
+                if not valid_resource_owner:
+                    request.resource_owner_key = self.dummy_access_token
         else:
             valid_resource_owner = True
 
