@@ -1,3 +1,4 @@
+# coding=utf-8
 """
 oauthlib.oauth2.draft_25.errors
 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
@@ -8,6 +9,7 @@ from oauthlib.common import urlencode, add_params_to_uri
 
 
 class OAuth2Error(Exception):
+    error = None
 
     def __init__(self, description=None, uri=None, state=None, status_code=400):
         """
@@ -55,6 +57,32 @@ class OAuth2Error(Exception):
         return json.dumps(dict(self.twotuples))
 
 
+class TokenExpiredError(OAuth2Error):
+    error = 'token_expired'
+
+
+class InsecureTransportError(OAuth2Error):
+    error = 'insecure_transport'
+    description = 'OAuth 2 MUST utilize https.'
+
+
+class MismatchingStateError(OAuth2Error):
+    error = 'mismatching_state'
+    description = 'CSRF Warning! State not equal in request and response.'
+
+
+class MissingCodeError(OAuth2Error):
+    error = 'missing_code'
+
+
+class MissingTokenError(OAuth2Error):
+    error = 'missing_token'
+
+
+class MissingTokenTypeError(OAuth2Error):
+    error = 'missing_token_type'
+
+
 class FatalClientError(OAuth2Error):
     pass
 
@@ -120,7 +148,7 @@ class ServerError(OAuth2Error):
     error = 'server_error'
 
 
-class TemporarilyUnvailableError(OAuth2Error):
+class TemporarilyUnavailableError(OAuth2Error):
     """The authorization server is currently unable to handle the request
     due to a temporary overloading or maintenance of the server.
     (This error code is needed because a 503 Service Unavailable HTTP
@@ -171,3 +199,16 @@ class InvalidScopeError(OAuth2Error):
     scope granted by the resource owner.
     """
     error = 'invalid_scope'
+
+
+def raise_from_error(error, params=None):
+    import inspect
+    import sys
+    kwargs = {
+        'description': params.get('error_description'),
+        'uri': params.get('error_uri'),
+        'state': params.get('state')
+    }
+    for _, cls in  inspect.getmembers(sys.modules[__name__], inspect.isclass):
+        if cls.error == error:
+            raise cls(**kwargs)
