@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 """
 oauthlib.oauth2.draft_25.grant_types
 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
@@ -44,9 +45,31 @@ class RequestValidator(object):
         """
         raise NotImplementedError('Subclasses must implement this method.')
 
+    def authenticate_client_id(self, client_id, request, *args, **kwargs):
+        """Ensure client_id belong to a non-confidential client.
+
+        A non-confidential client is one that is not required to authenticate
+        through other means, such as using HTTP Basic.
+
+        Note, while not strictly necessary it can often be very convenient
+        to set request.client to the client object associated with the
+        given client_id.
+
+        :param request: oauthlib.common.Request
+        :rtype: True or False
+
+        Method is used by:
+            - Authorization Code Grant
+        """
+        raise NotImplementedError('Subclasses must implement this method.')
+
     def confirm_redirect_uri(self, client_id, code, redirect_uri, client, *args, **kwargs):
         """Ensure client is authorized to redirect to the redirect_uri requested.
 
+        If the client specifies a redirect_uri when obtaining code then
+        that redirect URI must be bound to the code and verified equal
+        in this method.
+
         All clients should register the absolute URIs of all URIs they intend
         to redirect to. The registration is outside of the scope of oauthlib.
 
@@ -202,7 +225,8 @@ class RequestValidator(object):
         """Ensure the authorization_code is valid and assigned to client.
 
         OBS! The request.user attribute should be set to the resource owner
-        associated with this authorization code.
+        associated with this authorization code. Similarly request.scopes and
+        request.state must also be set.
 
         :param client_id: Unicode client identifier
         :param code: Unicode authorization code
@@ -320,6 +344,8 @@ class RequestValidator(object):
 
 
 class GrantTypeBase(object):
+    error_uri = None
+    request_validator = None
 
     def create_authorization_response(self, request, token_handler):
         raise NotImplementedError('Subclasses must implement this method.')
@@ -440,7 +466,7 @@ class AuthorizationCodeGrant(GrantTypeBase):
 
         # OPTIONAL. As described in Section 3.1.2.
         # http://tools.ietf.org/html/rfc6749#section-3.1.2
-        log.debug('Validating redirction uri %s for client %s.',
+        log.debug('Validating redirection uri %s for client %s.',
                   request.redirect_uri, request.client_id)
         if request.redirect_uri is not None:
             request.using_default_redirect_uri = False
@@ -476,7 +502,7 @@ class AuthorizationCodeGrant(GrantTypeBase):
 
         if not self.request_validator.validate_response_type(request.client_id,
                 request.response_type, request):
-            log.debug('Client %s is not authorized to use resposne_type %s.',
+            log.debug('Client %s is not authorized to use response_type %s.',
                       request.client_id, request.response_type)
             raise errors.UnauthorizedClientError()
 
@@ -509,18 +535,18 @@ class AuthorizationCodeGrant(GrantTypeBase):
         # in Section 3.2.1.
         # http://tools.ietf.org/html/rfc6749#section-3.2.1
         if not self.request_validator.authenticate_client(request):
-            log.debug('Could not authenticate client, %r.', request)
-            raise errors.AccessDeniedError()
-
-        # REQUIRED, if the client is not authenticating with the
-        # authorization server as described in Section 3.2.1.
-        # http://tools.ietf.org/html/rfc6749#section-3.2.1
-        if (not request.client and not
-                self.request_validator.validate_client_id(
-                    request.client_id, request)):
-            log.debug('Client_id not provided for unauthenticated client, %r.',
-                      request)
-            raise errors.UnauthorizedClientError()
+            # REQUIRED, if the client is not authenticating with the
+            # authorization server as described in Section 3.2.1.
+            # http://tools.ietf.org/html/rfc6749#section-3.2.1
+            if not self.request_validator.authenticate_client_id(
+                    request.client_id, request):
+                log.debug('Client authentication failed, %r.', request)
+                raise errors.InvalidClientError()
+        else:
+            if not hasattr(request.client, 'client_id'):
+                raise NotImplementedError('Authenticate client must set the '
+                                          'request.client.client_id attribute '
+                                          'in authenticate_client.')
 
         # Ensure client is authorized use of this grant type
         self.validate_grant_type(request)
@@ -737,6 +763,8 @@ class ImplicitGrant(GrantTypeBase):
             log.debug('Using default redirect_uri %s.', request.redirect_uri)
             if not request.redirect_uri:
                 raise errors.MissingRedirectURIError(state=request.state)
+            if not is_absolute_uri(request.redirect_uri):
+                raise errors.InvalidRedirectURIError(state=request.state)
 
         # Then check for normal errors.
 
@@ -853,7 +881,15 @@ class ResourceOwnerPasswordCredentialsGrant(GrantTypeBase):
         try:
             if require_authentication:
                 log.debug('Authenticating client, %r.', request)
-                self.request_validator.authenticate_client(request)
+                if not self.request_validator.authenticate_client(request):
+                    log.debug('Client authentication failed, %r.', request)
+                    raise errors.InvalidClientError()
+                else:
+                    if not hasattr(request.client, 'client_id'):
+                        raise NotImplementedError(
+                                'Authenticate client must set the '
+                                'request.client.client_id attribute '
+                                'in authenticate_client.')
             else:
                 log.debug('Client authentication disabled, %r.', request)
             log.debug('Validating access token request, %r.', request)
@@ -883,6 +919,12 @@ class ResourceOwnerPasswordCredentialsGrant(GrantTypeBase):
         if not self.request_validator.validate_user(request.username,
                 request.password, request.client, request):
             raise errors.InvalidGrantError('Invalid credentials given.')
+        else:
+            if not hasattr(request.client, 'client_id'):
+                raise NotImplementedError(
+                        'Validate user must set the '
+                        'request.client.client_id attribute '
+                        'in authenticate_client.')
         log.debug('Authorizing access to user %r.', request.user)
 
         # Ensure client is authorized use of this grant type
@@ -943,8 +985,6 @@ class ClientCredentialsGrant(GrantTypeBase):
         .. _`Section 5.2`: http://tools.ietf.org/html/rfc6749#section-5.2
         """
         try:
-            log.debug('Authenticating client, %r.', request)
-            self.request_validator.authenticate_client(request)
             log.debug('Validating access token request, %r.', request)
             self.validate_token_request(request)
         except errors.OAuth2Error as e:
@@ -958,15 +998,23 @@ class ClientCredentialsGrant(GrantTypeBase):
 
     def validate_token_request(self, request):
         if not getattr(request, 'grant_type'):
-            raise errors.InvalidRequestError('Request is issing grant type.')
+            raise errors.InvalidRequestError('Request is missing grant type.')
 
         if not request.grant_type == 'client_credentials':
             raise errors.UnsupportedGrantTypeError()
 
+        log.debug('Authenticating client, %r.', request)
+        if not self.request_validator.authenticate_client(request):
+            log.debug('Client authentication failed, %r.', request)
+            raise errors.InvalidClientError()
+        else:
+            if not hasattr(request.client, 'client_id'):
+                raise NotImplementedError('Authenticate client must set the '
+                                          'request.client.client_id attribute '
+                                          'in authenticate_client.')
         # Ensure client is authorized use of this grant type
         self.validate_grant_type(request)
 
-        request.user = request.user or request.client.user
         log.debug('Authorizing access to user %r.', request.user)
         request.client_id = request.client_id or request.client.client_id
         self.validate_scopes(request)
