@@ -8,7 +8,7 @@ from oauthlib import common
 from oauthlib.common import Request
 from oauthlib.oauth2.draft25.errors import UnsupportedGrantTypeError
 from oauthlib.oauth2.draft25.errors import InvalidRequestError
-from oauthlib.oauth2.draft25.errors import UnauthorizedClientError
+from oauthlib.oauth2.draft25.errors import InvalidClientError
 from oauthlib.oauth2.draft25.errors import InvalidGrantError
 from oauthlib.oauth2.draft25.grant_types import AuthorizationCodeGrant
 from oauthlib.oauth2.draft25.grant_types import ImplicitGrant
@@ -51,8 +51,14 @@ class AuthorizationCodeGrantTest(TestCase):
         self.request_state.state = 'abc'
 
         self.mock_validator = mock.MagicMock()
+        self.mock_validator.authenticate_client.side_effect = self.set_client
         self.auth = AuthorizationCodeGrant(request_validator=self.mock_validator)
 
+    def set_client(self, request):
+        request.client = mock.MagicMock()
+        request.client.client_id = 'mocked'
+        return True
+
     def test_create_authorization_grant(self):
         grant = self.auth.create_authorization_code(self.request)
         self.assertIn('code', grant)
@@ -81,14 +87,15 @@ class AuthorizationCodeGrantTest(TestCase):
         self.assertRaises(InvalidRequestError,
                 auth.validate_token_request, request)
 
-        mock_validator.validate_client = mock.MagicMock(return_value=False)
+        mock_validator.authenticate_client.return_value = False
+        mock_validator.authenticate_client_id.return_value = False
         request.code = 'waffles'
-        request.client = 'batman'
-        self.assertRaises(UnauthorizedClientError,
+        self.assertRaises(InvalidClientError,
                 auth.validate_token_request, request)
 
-        mock_validator.validate_client = mock.MagicMock(return_value=True)
-        mock_validator.validate_code = mock.MagicMock(return_value=False)
+        request.client = 'batman'
+        mock_validator.authenticate_client = self.set_client
+        mock_validator.validate_code.return_value = False
         self.assertRaises(InvalidGrantError,
                 auth.validate_token_request, request)
 
@@ -96,9 +103,11 @@ class AuthorizationCodeGrantTest(TestCase):
 class ImplicitGrantTest(TestCase):
 
     def setUp(self):
+        mock_client = mock.MagicMock()
+        mock_client.user.return_value = 'mocked user'
         self.request = Request('http://a.b/path')
         self.request.scopes = ('hello', 'world')
-        self.request.client = 'batman'
+        self.request.client = mock_client
         self.request.client_id = 'abcdef'
         self.request.response_type = 'token'
         self.request.state = 'xyz'
@@ -114,7 +123,7 @@ class ImplicitGrantTest(TestCase):
         common.generate_token = lambda *args, **kwargs: '1234'
         uri, headers, body, status_code = self.auth.create_token_response(
                 self.request, bearer)
-        correct_uri = 'https://b.c/p#access_token=1234&token_type=Bearer&expires_in=3600&state=xyz&scope=hello+world'
+        correct_uri = 'https://b.c/p#access_token=1234&token_type=Bearer&expires_in=3600&state=xyz'
         self.assertURLEqual(uri, correct_uri, parse_fragment=True)
 
     def test_error_response(self):
@@ -124,11 +133,13 @@ class ImplicitGrantTest(TestCase):
 class ResourceOwnerPasswordCredentialsGrantTest(TestCase):
 
     def setUp(self):
+        mock_client = mock.MagicMock()
+        mock_client.user.return_value = 'mocked user'
         self.request = Request('http://a.b/path')
         self.request.grant_type = 'password'
         self.request.username = 'john'
         self.request.password = 'doe'
-        self.request.client = 'mock authenticated'
+        self.request.client = mock_client
         self.request.scopes = ('mocked', 'scopes')
         self.mock_validator = mock.MagicMock()
         self.auth = ResourceOwnerPasswordCredentialsGrant(
@@ -154,9 +165,11 @@ class ResourceOwnerPasswordCredentialsGrantTest(TestCase):
 class ClientCredentialsGrantTest(TestCase):
 
     def setUp(self):
+        mock_client = mock.MagicMock()
+        mock_client.user.return_value = 'mocked user'
         self.request = Request('http://a.b/path')
         self.request.grant_type = 'client_credentials'
-        self.request.client = 'mock authenticated'
+        self.request.client = mock_client
         self.request.scopes = ('mocked', 'scopes')
         self.mock_validator = mock.MagicMock()
         self.auth = ClientCredentialsGrant(
